---
title: llarp::layers::platform::PlatformDriver
summary: base type that is responsible for bridging the os and the flow layer. 

---

# llarp::layers::platform::PlatformDriver



base type that is responsible for bridging the os and the flow layer.  [More...](#detailed-description)


`#include <platform_driver.hpp>`

## Public Functions

|                | Name           |
| -------------- | -------------- |
| | **[PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-platformdriver)**([PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md) * ) |
| void | **[wakeup_os](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-wakeup-os)**() const<br>wake up lokinet event loop telling it we sent some things to the network.  |
| void | **[wakeup_flow_layer](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-wakeup-flow-layer)**() const<br>wake up the flow layer. tell it we sent it some things and should process it.  |
| std::shared_ptr< [NameResolverJob_Base](Classes/classllarp_1_1layers_1_1platform_1_1NameResolverJob__Base.md) > | **[make_resolver_job_for](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-make-resolver-job-for)**(UnresolvedName_t name) const<br>given a name we want to look up, make a one shot job that we will use to look it up from the network.  |
| void | **[map_remote](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-map-remote)**(std::string name, std::string auth, std::vector< [IPRange](Classes/structllarp_1_1IPRange.md) > ranges, std::function< void(std::optional< [flow::FlowInfo](Classes/structllarp_1_1layers_1_1flow_1_1FlowInfo.md) >, std::string)> result_handler) |
| void | **[map_exit](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-map-exit)**(std::string name, std::string auth, std::vector< [IPRange](Classes/structllarp_1_1IPRange.md) > ranges, std::function< void(bool, std::string)> result_handler)<br>calls map_remote() for exits  |

## Protected Functions

|                | Name           |
| -------------- | -------------- |
| void | **[before_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-before-loop-ready)**()<br>called once per event loop tick, before polling for io.  |
| bool | **[loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-loop-ready)**()<br>called once per event loop tick after platform layer event loop is ready to be read.  |
| void | **[after_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-after-loop-ready)**()<br>called once per event loop after we have processed all our events from a read cycle.  |

## Protected Attributes

|                | Name           |
| -------------- | -------------- |
| std::unique_ptr< [PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md) > | **[_impl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--impl)**  |

## Detailed Description

```cpp
class llarp::layers::platform::PlatformDriver;
```

base type that is responsible for bridging the os and the flow layer. 

each kind of ethertype has its own implementation. 

## Public Functions Documentation

### function PlatformDriver

```cpp
PlatformDriver(
    PlatformDriver_pimpl * 
)
```


### function wakeup_os

```cpp
void wakeup_os() const
```

wake up lokinet event loop telling it we sent some things to the network. 

### function wakeup_flow_layer

```cpp
void wakeup_flow_layer() const
```

wake up the flow layer. tell it we sent it some things and should process it. 

### function make_resolver_job_for

```cpp
std::shared_ptr< NameResolverJob_Base > make_resolver_job_for(
    UnresolvedName_t name
) const
```

given a name we want to look up, make a one shot job that we will use to look it up from the network. 

### function map_remote

```cpp
void map_remote(
    std::string name,
    std::string auth,
    std::vector< IPRange > ranges,
    std::function< void(std::optional< flow::FlowInfo >, std::string)> result_handler
)
```


### function map_exit

```cpp
void map_exit(
    std::string name,
    std::string auth,
    std::vector< IPRange > ranges,
    std::function< void(bool, std::string)> result_handler
)
```

calls map_remote() for exits 

## Protected Functions Documentation

### function before_loop_ready

```cpp
void before_loop_ready()
```

called once per event loop tick, before polling for io. 

### function loop_ready

```cpp
bool loop_ready()
```

called once per event loop tick after platform layer event loop is ready to be read. 

returns true if we proccessed events and thus need to call [after_loop_ready()](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-after-loop-ready). 


### function after_loop_ready

```cpp
void after_loop_ready()
```

called once per event loop after we have processed all our events from a read cycle. 

## Protected Attributes Documentation

### variable _impl

```cpp
std::unique_ptr< PlatformDriver_pimpl > _impl;
```


-------------------------------

Updated on 2023-01-27 at 20:09:05 +0000