---
title: llarp/layers/platform/deprecated_driver_pimpl.hpp

---

# llarp/layers/platform/deprecated_driver_pimpl.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::platform::NameLookupJob_Deprecated](Classes/classllarp_1_1layers_1_1platform_1_1NameLookupJob__Deprecated.md)**  |
| class | **[llarp::layers::platform::PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md)**  |
| class | **[llarp::layers::platform::OSTraffic_IO_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO__pimpl.md)**  |
| class | **[llarp::layers::platform::FlowTraffic_IO_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1FlowTraffic__IO__pimpl.md)**  |




## Source code

```cpp
#pragma once

#include "addr_mapper.hpp"
#include "ethertype.hpp"
#include "platform_addr.hpp"
#include "name_resolver.hpp"
#include "traffic.hpp"
#include "platform_driver.hpp"

#include <llarp/layers/flow/flow_layer.hpp>

#include <llarp/dns/question.hpp>
#include <llarp/router/router.hpp>

#include <stdexcept>
#include <oxenc/variant.h>

namespace llarp::layers::platform
{

  class NameLookupJob_Deprecated : public NameResolverJob_Base
  {
    std::shared_ptr<EndpointBase> _endpoint;

   public:
    NameLookupJob_Deprecated(
        std::shared_ptr<EndpointBase> endpoint_,
        std::string_view name_,
        std::chrono::seconds timeout_)
        : NameResolverJob_Base{name_, timeout_}, _endpoint{std::move(endpoint_)}
    {}

    void
    resolve_flow_addr_async(
        std::function<void(std::optional<flow::FlowAddr>)> handle_result) override
    {
      _endpoint->LookupNameAsync(name, [handle_result = std::move(handle_result)](auto maybe_addr) {
        if (not maybe_addr)
        {
          handle_result(std::nullopt);
          return;
        }

        flow::FlowAddr found{var::visit([](auto&& x) { return "{}"_format(x); }, *maybe_addr)};
        handle_result(found);
      });
    }
  };

  class PlatformDriver_pimpl
  {
    AbstractRouter& _router;

   public:
    AddrMapper addr_mapper;

    flow::FlowLayer_Base&
    flow_layer()
    {
      return *_router.get_layers()->flow;
    }

    std::shared_ptr<NameResolverJob_Base>
    make_resolver_job_for(UnresolvedName_t name)
    {
      if (auto* dns_ptr = std::get_if<dns::Question>(name))
        throw std::invalid_argument{
            "cannot resolve '{}': icann dns name queries not implemented rn"_format(
                dns_ptr->qname)};

      return std::make_shared<NameLookupJob_Deprecated>(
          _router.get_default_endpoint(), std::get<std::string_view>(name), 0s);
    }

    bool
    put_exit_up()
    {
      bool has_exit = _router.HasClientExit();
      if (const auto& poker = _router.routePoker(); poker and has_exit)
        poker->Up();

      return has_exit;
    }

    void
    put_exit_down()
    {
      if (const auto& poker = _router.routePoker())
        poker->Down();
    }

    PlatformDriver_pimpl(AbstractRouter& router) : _router{router}
    {}
  };

  class OSTraffic_IO_pimpl
  {
    std::shared_ptr<vpn::NetworkInterface> _netif;

   public:
    OSTraffic_IO_pimpl(std::shared_ptr<vpn::NetworkInterface> network_if)
        : _netif{std::move(network_if)}
    {}

    std::vector<OSTraffic>
    read_platform_traffic()
    {
      std::vector<OSTraffic> pkts;
      const bool has_vpn = _netif != nullptr;
      while (has_vpn)
      {
        OSTraffic os_pkt{};
        os_pkt.kind = EtherType_t::ip_unicast;

        auto pkt = _netif->ReadNextPacket();
        if (pkt.empty())
          break;
        os_pkt.src.ip = net::ipv6addr_t::from_host(pkt.srcv6());
        os_pkt.dst.ip = net::ipv6addr_t::from_host(pkt.dstv6());
        os_pkt.data = pkt.steal();
        pkts.push_back(std::move(os_pkt));
      }
      return pkts;
    }

    void
    write_platform_traffic(std::vector<OSTraffic>&& traff)
    {
      if (not _netif)
        return;
      for (auto& os_pkt : traff)
      {
        net::IPPacket pkt{std::move(os_pkt.data)};
        if (pkt.empty())
          continue;
        if (pkt.IsV4())
          pkt.UpdateIPv4Address(os_pkt.src.as_v4(), os_pkt.dst.as_v4());
        else if (pkt.IsV6())
          pkt.UpdateIPv6Address(ToHost(os_pkt.src.ip), ToHost(os_pkt.dst.ip));
        if (os_pkt.kind == EtherType_t::ip_unicast and _netif)
          _netif->WritePacket(pkt);
      }
    }
  };

  class FlowTraffic_IO_pimpl
  {};
}  // namespace llarp::layers::platform
```


-------------------------------

Updated on 2023-01-27 at 20:09:05 +0000
